@@ -170,7 +170,7 @@ impl<'a, T: 'a, const N: usize> Iterator for StaticVecIterConst<'a, T, N> {
     // Safety: see the comments for the implementation of this function in
     // 'core/slice/iter/macros.rs`, as they're equally applicable to us here.
     match size_of::<T>() {
-      0 => &*zst_ptr_add(self.start, idx),
+      0 => &*((self.start as usize + idx) as *const T),
       _ => &*self.start.add(idx),
     }
   }
@@ -242,6 +242,7 @@ unsafe impl<'a, T: 'a, const N: usize> const TrustedRandomAccessNoCoerce
   }
 }
 unsafe impl<'a, T: 'a + Sync, const N: usize> const Sync for StaticVecIterConst<'a, T, N> {}
+// `StaticVecIterConst` works in terms of `&T`, which is sendable if / when `T` is `Sync`.
 unsafe impl<'a, T: 'a + Sync, const N: usize> const Send for StaticVecIterConst<'a, T, N> {}
 
 impl<'a, T: 'a, const N: usize> Clone for StaticVecIterConst<'a, T, N> {
@@ -367,7 +368,7 @@ impl<'a, T: 'a, const N: usize> Iterator for StaticVecIterMut<'a, T, N> {
     // Safety: see the comments for the implementation of this function in
     // 'core/slice/iter/macros.rs`, as they're equally applicable to us here.
     match size_of::<T>() {
-      0 => &mut *zst_ptr_add_mut(self.start, idx),
+      0 => &mut *((self.start as usize + idx) as *mut T),
       _ => &mut *self.start.add(idx),
     }
   }
@@ -648,11 +649,24 @@ unsafe impl<T: Send, const N: usize> const Send for StaticVecIntoIter<T, N> {}
 impl<T: Clone, const N: usize> Clone for StaticVecIntoIter<T, N> {
   #[inline(always)]
   fn clone(&self) -> StaticVecIntoIter<T, N> {
-    let mut res = StaticVec::new();
-    for item in self.as_slice() {
-      unsafe { res.push_unchecked(item.clone()) };
+    Self {
+      start: self.start,
+      end: self.end,
+      data: {
+        let mut data = MaybeUninit::<[T; N]>::uninit();
+        let new_data_ptr = data.as_mut_ptr() as *mut T;
+        let self_data_ptr = self.data.as_ptr() as *const T;
+        unsafe {
+          // These are guaranteed safe assumptions in this context.
+          assume(!new_data_ptr.is_null());
+          assume(!self_data_ptr.is_null());
+          for i in self.start..self.end {
+            new_data_ptr.add(i).write((&*self_data_ptr.add(i)).clone());
+          }
+        }
+        data
+      },
     }
-    res.into_iter()
   }
 }
 
