@@ -31,6 +31,7 @@
 #![feature(
   adt_const_params,
   const_assume,
+  const_eval_select,
   const_fn_floating_point_arithmetic,
   const_fn_fn_ptr_basics,
   const_fn_trait_bound,
@@ -83,8 +84,8 @@ pub use crate::heap::{
 pub use crate::iterators::{
   StaticVecDrain, StaticVecIntoIter, StaticVecIterConst, StaticVecIterMut, StaticVecSplice,
 };
-pub use crate::string::{string_utils, StaticString, StringError};
-use crate::utils::{const_min, quicksort_internal, reverse_copy};
+pub use crate::string::{StaticString, StringError};
+use crate::utils::{const_min, quicksort_internal, reverse_copy, zst_ptr_add, zst_ptr_add_mut};
 
 #[cfg(any(feature = "std", rustdoc))]
 extern crate alloc;
@@ -95,16 +96,13 @@ use alloc::vec::Vec;
 #[cfg(feature = "std")]
 extern crate std;
 
+mod errors;
+mod heap;
 mod iterators;
 #[macro_use]
 mod macros;
-#[doc(hidden)]
-mod errors;
-#[doc(hidden)]
-mod heap;
-#[doc(hidden)]
 mod string;
-pub(crate) mod trait_impls;
+mod trait_impls;
 #[doc(hidden)]
 pub mod utils;
 
@@ -204,13 +202,24 @@ impl<T, const N: usize> StaticVec<T, N> {
   /// let v2 = StaticVec::<i32, 3>::new_from_array([1, 2, 3, 4, 5, 6]);
   /// assert_eq!(v2, [1, 2, 3]);
   /// ```
-  /// Note that StaticVec also implements [`From`](core::convert::From) for both slices and static
-  /// arrays (as well as several other types), which may prove more ergonomic in some cases as it
-  /// allows for a greater degree of type inference:
+  /// Note that StaticVec also implements [`From`](core::convert::From) for both slices and
+  /// fixed-size arrays (as well as several other types), which may prove more ergonomic in some
+  /// cases as it allows for a greater degree of type inference:
   /// ```
+  /// # #![feature(box_syntax)]
   /// # use staticvec::StaticVec;
   /// // The StaticVec on the next line is inferred to be of type `StaticVec<&'static str, 4>`.
   /// let v = StaticVec::from(["A", "B", "C", "D"]);
+  /// assert_eq!(v, ["A", "B", "C", "D"]);
+  /// // You can essentially use [`From`](core::convert::From) in the vast majority of scenarios
+  /// // where *either* [`new_from_slice`](crate::StaticVec::new_from_slice) or
+  /// // [`new_from_array`](crate::StaticVec::new_from_array) would be accepted by the compiler,
+  /// // and expect it to "just work" like the appropriate one of the two on a contextual basis.
+  /// // For example, the two "extra" instances of [`Box`](std::boxed::Box) on the next line will
+  /// // be correctly dropped just as they would be when calling
+  /// // [`new_from_array`](crate::StaticVec::new_from_array).  
+  /// let v2 = StaticVec::<Box<i32>, 2>::from([box 1, box 2, box 3, box 4]);
+  /// assert_eq!(v2, [box 1, box 2]);
   /// ```
   #[inline]
   pub fn new_from_array<const N2: usize>(values: [T; N2]) -> Self {
@@ -983,6 +992,7 @@ impl<T, const N: usize> StaticVec<T, N> {
     );
     unsafe {
       let self_ptr = self.mut_ptr_at_unchecked(index);
+      assume(!self_ptr.is_null());
       let res = self_ptr.read();
       self_ptr.offset(1).copy_to(self_ptr, old_length - index - 1);
       self.set_len(old_length - 1);
@@ -1293,7 +1303,7 @@ impl<T, const N: usize> StaticVec<T, N> {
       StaticVecIterConst {
         start: start_ptr,
         end: match size_of::<T>() {
-          0 => (start_ptr as *const u8).wrapping_add(self.length) as *const T,
+          0 => zst_ptr_add(start_ptr, self.length),
           _ => start_ptr.add(self.length),
         },
         marker: PhantomData,
@@ -1322,7 +1332,7 @@ impl<T, const N: usize> StaticVec<T, N> {
       StaticVecIterMut {
         start: start_ptr,
         end: match size_of::<T>() {
-          0 => (start_ptr as *mut u8).wrapping_add(self.length) as *mut T,
+          0 => zst_ptr_add_mut(start_ptr, self.length),
           _ => start_ptr.add(self.length),
         },
         marker: PhantomData,
@@ -1981,7 +1991,10 @@ impl<T, const N: usize> StaticVec<T, N> {
     unsafe {
       // Set the length to `start` to avoid memory issues if anything goes wrong with the Drain.
       self.set_len(start);
-      let start_ptr = self.ptr_at_unchecked(start);
+      let start_ptr = match size_of::<T>() {
+        0 => zst_ptr_add(self.as_ptr(), start),
+        _ => self.ptr_at_unchecked(start),
+      };
       // `start_ptr` will never be null, so this is a safe assumption to give to
       // the optimizer.
       assume(!start_ptr.is_null());
@@ -1992,7 +2005,7 @@ impl<T, const N: usize> StaticVec<T, N> {
         iter: StaticVecIterConst {
           start: start_ptr,
           end: match size_of::<T>() {
-            0 => (self.as_ptr() as *const u8).wrapping_add(end) as *const T,
+            0 => zst_ptr_add(self.as_ptr(), end),
             _ => self.ptr_at_unchecked(end),
           },
           marker: PhantomData,
@@ -2135,7 +2148,7 @@ impl<T, const N: usize> StaticVec<T, N> {
   /// Splits one StaticVec into two at the given index, returning the second half without consuming
   /// the first half. The original StaticVec will contain all elements within the exclusive range
   /// `0..at`, and the new one will contain all elements within the exclusive range
-  /// `at..self.len()`.
+  /// `at..self.len()`. This function will panic if `at` is greater than `self.len()`.
   ///
   /// # Example usage:
   /// ```
@@ -2169,7 +2182,7 @@ impl<T, const N: usize> StaticVec<T, N> {
   /// Splits one StaticVec into two new ones at index `M` and returns them in a tuple, while
   /// consuming the original. The first new one will contain all elements within the exclusive range
   /// `0..M`, and the second new one will contain all elements within the exclusive range
-  /// `M..self.len()`.
+  /// `M..self.len()`. This function will panic if `M` is greater than `self.len()`.
   ///
   /// # Example usage:
   /// ```
